@@ -2,6 +2,7 @@ import ast, sys, json, os, time, base64
 import requests
 import random, copy
 from collections import OrderedDict
+from hwilib.descriptor import AddChecksum, Descriptor
 from mnemonic import Mnemonic
 from threading import Thread
 from .key import Key
@@ -27,6 +28,8 @@ from datetime import datetime
 import urllib
 from io import BytesIO
 import traceback
+from .devices.electrum import b43_decode
+from binascii import b2a_base64
 
 from pathlib import Path
 env_path = Path('.') / '.flaskenv'
@@ -65,25 +68,57 @@ def combine(wallet_alias):
     except SpecterError as se:
         app.logger.error("SpecterError while combine: %s" % se)
         return render_template("base.jinja", error=se, specter=app.specter, rand=rand)
-    if request.method == 'POST': # FIXME: ugly...
-        psbt0 = request.form.get('psbt0') # request.args.get('psbt0')
-        psbt1 = request.form.get('psbt1') # request.args.get('psbt1')
-        if "UR:BYTES/" in psbt0:
-            psbt0 = bcur2base64(psbt0)
-        if "UR:BYTES/" in psbt1:
-            psbt1 = bcur2base64(psbt1)
+    if request.method == 'POST': 
+        # FIXME: ugly...
         txid = request.form.get('txid')
+        psbts = [
+            request.form.get('psbt0').strip(),
+            request.form.get('psbt1').strip()
+        ]
+        raw = {}
+        combined = None
 
-        try:
-            psbt = app.specter.combine([psbt0, psbt1])
-            raw = app.specter.finalize(psbt)
-            if "psbt" not in raw:
-                raw["psbt"] = psbt
-        except RpcError as e:
-            return e.error_msg, e.status_code
-        except Exception as e:
-            return "Unknown error: %r" % e, 500
-        psbt = wallet.update_pending_psbt(psbt, txid, raw)
+        for i, psbt in enumerate(psbts):
+            if "UR:BYTES/" in psbt.upper():
+                psbt = bcur2base64(psbt).decode()
+
+            # if electrum then it's base43
+            try:
+                decoded = b43_decode(psbt)
+                if decoded.startswith(b"psbt\xff"):
+                    psbt = b2a_base64(decoded).decode()
+                else:
+                    psbt = decoded.hex()
+            except:
+                pass
+
+            psbts[i] = psbt
+            # psbt should start with cHNi
+            # if not - maybe finalized hex tx
+            if not psbt.startswith("cHNi"):
+                raw["hex"] = psbt
+                combined = psbts[1-i]
+
+        # try converting to bytes
+        if "hex" in raw:
+            raw["complete"] = True
+            raw["psbt"] = combined
+            try:
+                bytes.fromhex(raw["hex"])
+            except:
+                return "Invalid transaction format", 500
+
+        else:
+            try:
+                combined = app.specter.combine(psbts)
+                raw = app.specter.finalize(combined)
+                if "psbt" not in raw:
+                    raw["psbt"] = combined
+            except RpcError as e:
+                return e.error_msg, e.status_code
+            except Exception as e:
+                return "Unknown error: %r" % e, 500
+        psbt = wallet.update_pending_psbt(combined, txid, raw)
         devices = []
         # we get names, but need aliases
         if "devices_signed" in psbt:
@@ -421,7 +456,7 @@ def new_wallet_type():
 @app.route('/new_wallet/<wallet_type>/', methods=['GET', 'POST'])
 @login_required
 def new_wallet(wallet_type):
-    wallet_types = ['simple', 'multisig']
+    wallet_types = ['simple', 'multisig', 'import_wallet']
     if wallet_type not in wallet_types:
         err = "Unknown wallet type requested"
         return render_template("base.jinja", specter=app.specter, rand=rand)
@@ -448,21 +483,123 @@ def new_wallet(wallet_type):
 
     if request.method == 'POST':
         action = request.form['action']
-        wallet_name = request.form['wallet_name']
-        if wallet_name in app.specter.wallet_manager.wallets_names:
-            err = "Wallet already exists"
-        address_type = request.form['type']
-        pur = {
-            '': "General",
-            "wpkh": "Segwit (bech32)",
-            "sh-wpkh": "Nested Segwit",
-            "pkh": "Legacy",
-            "wsh": "Segwit (bech32)",
-            "sh-wsh": "Nested Segwit",
-            "sh": "Legacy",
-        }
-        sigs_total = int(request.form.get('sigs_total', 1))
-        sigs_required = int(request.form.get('sigs_required', 1))
+        if action == "importwallet":
+            wallet_data = json.loads(request.form['wallet_data'].replace("'", "h"))
+            wallet_name = wallet_data['label'] if 'label' in wallet_data else 'Imported Wallet'
+            startblock = wallet_data['blockheight'] if 'blockheight' in wallet_data else app.specter.wallet_manager.cli.getblockcount()
+            try:
+                descriptor = Descriptor.parse(AddChecksum(wallet_data['descriptor'].split('#')[0]), testnet=app.specter.chain != 'main')
+                if descriptor is None:
+                    err = "Invalid wallet descriptor."
+            except:
+                err = "Invalid wallet descriptor."
+            if wallet_name in app.specter.wallet_manager.wallets_names:
+                err = "Wallet with the same name already exists"
+
+            if not err:
+                try:
+                    sigs_total = descriptor.multisig_N
+                    sigs_required = descriptor.multisig_M
+                    if descriptor.wpkh:
+                        address_type = 'wpkh'
+                    elif descriptor.wsh:
+                        address_type = 'wsh'
+                    elif descriptor.sh_wpkh:
+                        address_type = 'sh-wpkh'
+                    elif descriptor.sh_wsh:
+                        address_type = 'sh-wsh'
+                    elif descriptor.sh:
+                        address_type = 'sh-wsh'
+                    else:
+                        address_type = 'pkh'
+                    keys = []
+                    cosigners = []
+                    unknown_cosigners = []
+                    if sigs_total == None:
+                        sigs_total = 1
+                        sigs_required = 1
+                        descriptor.origin_fingerprint = [descriptor.origin_fingerprint]
+                        descriptor.origin_path = [descriptor.origin_path]
+                        descriptor.base_key = [descriptor.base_key]
+                    for i in range(sigs_total):
+                        cosigner_found = False
+                        for device in app.specter.device_manager.devices:
+                            cosigner = app.specter.device_manager.devices[device]
+                            for key in cosigner.keys:
+                                if key.fingerprint + key.derivation.replace('m', '') == \
+                                    descriptor.origin_fingerprint[i] + descriptor.origin_path[i].replace("'", 'h'):
+                                    keys.append(key)
+                                    cosigners.append(cosigner)
+                                    cosigner_found = True
+                                    break
+                            if cosigner_found:
+                                break
+                        if not cosigner_found:
+                            desc_key = Key.parse_xpub('[{}{}]{}'.format(
+                                descriptor.origin_fingerprint[i],
+                                descriptor.origin_path[i],
+                                descriptor.base_key[i],
+                            ))
+                            unknown_cosigners.append(desc_key)
+                        #     raise Exception('Could not find device with matching key to import wallet')
+                    wallet_type = 'multisig' if sigs_total > 1 else 'simple'
+                    createwallet = 'createwallet' in request.form
+                    if createwallet:
+                        wallet_name = request.form['wallet_name']
+                        for i, unknown_cosigner in enumerate(unknown_cosigners):
+                            unknown_cosigner_name = request.form['unknown_cosigner_{}_name'.format(i)]
+                            device = app.specter.device_manager.add_device(name=unknown_cosigner_name, device_type='other', keys=[unknown_cosigner])
+                            keys.append(unknown_cosigner)
+                            cosigners.append(device)
+                        wallet = app.specter.wallet_manager.create_wallet(wallet_name, sigs_required, address_type, keys, cosigners)
+                        flash("Wallet imported successfully", "info")
+                        try:
+                            wallet.cli.rescanblockchain(startblock, timeout=1)
+                            app.logger.info("Rescanning Blockchain ...")
+                        except requests.exceptions.ReadTimeout:
+                            # this is normal behavior in our usecase
+                            pass
+                        except Exception as e:
+                            app.logger.error("Exception while rescanning blockchain: %e" % e)
+                            flash("Failed to perform rescan for wallet: %r" % e, 'error')
+                        wallet.getdata()
+                        return redirect("/wallets/%s/" % wallet.alias)
+                    else:
+                        return render_template(
+                            "wallet/new_wallet/import_wallet.jinja",
+                            wallet_data=json.dumps(wallet_data),
+                            wallet_type=wallet_type,
+                            wallet_name=wallet_name,
+                            cosigners=cosigners,
+                            unknown_cosigners=unknown_cosigners,
+                            sigs_required=sigs_required,
+                            sigs_total=sigs_total,
+                            error=err,
+                            specter=app.specter,
+                            rand=rand
+                        )
+                except Exception as e:
+                    err = "%r" % e
+
+            if err:
+                return render_template("wallet/new_wallet/new_wallet_type.jinja", error="Failed to import wallet: " + err, specter=app.specter, rand=rand)
+        else:
+            wallet_name = request.form['wallet_name']
+            if wallet_name in app.specter.wallet_manager.wallets_names:
+                err = "Wallet already exists"
+            address_type = request.form['type']
+            pur = {
+                '': "General",
+                "wpkh": "Segwit (bech32)",
+                "sh-wpkh": "Nested Segwit",
+                "pkh": "Legacy",
+                "wsh": "Segwit (bech32)",
+                "sh-wsh": "Nested Segwit",
+                "sh": "Legacy",
+            }
+            sigs_total = int(request.form.get('sigs_total', 1))
+            sigs_required = int(request.form.get('sigs_required', 1))
+
         if action == 'device' and err is None:
             cosigners = [app.specter.device_manager.get_by_alias(alias) for alias in request.form.getlist('devices')]
             if len(cosigners) != sigs_total:
@@ -845,7 +982,7 @@ def wallet_settings(wallet_alias):
 def new_device():
     app.specter.check()
     err = None
-    device_type = "other"
+    device_type = ""
     device_name = ""
     xpubs = ""
     strength = 128
@@ -887,7 +1024,10 @@ def new_device():
         elif action == 'generatemnemonic':
             strength = int(request.form['strength'])
             mnemonic = generate_mnemonic(strength=strength)
-    return render_template("device/new_device.jinja", device_type=device_type, device_name=device_name, xpubs=xpubs, mnemonic=mnemonic, strength=strength, error=err, specter=app.specter, rand=rand)
+    return render_template("device/new_device.jinja", device_type=device_type, 
+                            device_name=device_name, xpubs=xpubs, 
+                            mnemonic=mnemonic, strength=strength, 
+                            error=err, specter=app.specter, rand=rand)
 
 @app.route('/devices/<device_alias>/', methods=['GET', 'POST'])
 @login_required
