@@ -1,6 +1,6 @@
 import copy, hashlib, json, logging, os
 from time import time
-from .descriptor import AddChecksum
+from hwilib.descriptor import AddChecksum
 from .device import Device
 from .key import Key
 from .helpers import decode_base58, der_to_bytes, get_xpub_fingerprint, sort_descriptor, fslock
@@ -261,6 +261,16 @@ class Wallet():
         else:
             return self.info["scanning"]["progress"]
 
+    @property
+    def blockheight(self):
+        if len(self.transactions) > 0 and 'block_height' in self.transactions[0]:
+            return self.transactions[0]['block_height'] - 101 # To ensure coinbase transactions are indexed properly
+        return self.cli.getblockcount()
+
+    @property
+    def account_map(self):
+        return '{ "label": "' + self.name + '", "blockheight": ' + str(self.blockheight) + ', "descriptor": "' + self.recv_descriptor.replace("/", "\\/") + '" }'
+
     def getnewaddress(self, change=False):
         label = "Change" if change else "Address"
         if change:
@@ -529,19 +539,35 @@ class Wallet():
 
         return psbt
 
-    def fill_psbt(self, b64psbt):
+    def fill_psbt(self, b64psbt, non_witness:bool=True, xpubs:bool=True):
         psbt = PSBT()
         psbt.deserialize(b64psbt)
-        for i, inp in enumerate(psbt.tx.vin):
-            txid = inp.prevout.hash.to_bytes(32,'big').hex()
-            try:
-                res = self.cli.gettransaction(txid)
-            except:
-                raise SpecterError("Can't find previous transaction in the wallet.")
-            stream = BytesIO(bytes.fromhex(res["hex"]))
-            prevtx = CTransaction()
-            prevtx.deserialize(stream)
-            psbt.inputs[i].non_witness_utxo = prevtx
+        if non_witness:
+            for i, inp in enumerate(psbt.tx.vin):
+                txid = inp.prevout.hash.to_bytes(32,'big').hex()
+                try:
+                    res = self.cli.gettransaction(txid)
+                except:
+                    raise SpecterError("Can't find previous transaction in the wallet.")
+                stream = BytesIO(bytes.fromhex(res["hex"]))
+                prevtx = CTransaction()
+                prevtx.deserialize(stream)
+                psbt.inputs[i].non_witness_utxo = prevtx
+        if xpubs:
+            # for multisig add xpub fields
+            if len(self.keys) > 1:
+                for k in self.keys:
+                    key = b'\x01' + decode_base58(k.xpub)
+                    if k.fingerprint != '':
+                        fingerprint = bytes.fromhex(k.fingerprint)
+                    else:
+                        fingerprint = get_xpub_fingerprint(k.xpub)
+                    if k.derivation != '':
+                        der = der_to_bytes(k.derivation)
+                    else:
+                        der = b''
+                    value = fingerprint + der
+                    psbt.unknown[key] = value
         return psbt.serialize()
 
     def get_signed_devices(self, decodedpsbt):
